#include "hmc6352.h"
#include "usart.h"
#include "delay.h"
#include "i2c.h"

// HW Layer
void _HMCPins(void)
{
	// RB0:SDA
	// RB1:SCL
	// RB2:GND
	// RB3:VCC

	dbg("InitPins\n");
	ANSELH = 0;
	TRISB2 = 0;
	TRISB3 = 0;
	RB2 = 0;
	RB3 = 1;
}

// HMC interface
#define HMC6352_I2C_WRITE_ADDRESS 0x42
#define HMC6352_I2C_READ_ADDRESS  0x43
#define HMC6352_GET_DATA_COMMAND 0x41 
#define HMC6352_RAM_WRITE_COMMAND 'G'
#define HMC6352_ENTER_CALIBRATION_COMMAND 'C'
#define HMC6352_EXIT_CALIBRATION_COMMAND 'E'

BOOL HMCInit(void)
{
	dbg("HMCInit()\n");
	_HMCPins();
	i2c_init();
	return TRUE;
}

void HMCSetMode(byte bMode)
{
	i2c_start();
	_ASSERT( i2c_write(HMC6352_I2C_WRITE_ADDRESS) == 0 );
	_ASSERT( i2c_write(HMC6352_RAM_WRITE_COMMAND) == 0 );
	_ASSERT( i2c_write(0x4E) == 0 ); // 0x4E - Output Mode control byte
	_ASSERT( i2c_write(bMode) == 0 );
	i2c_stop(); 
}

void HMCEnterCalibration(void)
{
	i2c_start();
	_ASSERT( i2c_write(HMC6352_I2C_WRITE_ADDRESS) == 0 );
	_ASSERT( i2c_write(HMC6352_ENTER_CALIBRATION_COMMAND) == 0 );
	i2c_stop(); 
}

void HMCExitCalibration(void)
{
	i2c_start();
	_ASSERT( i2c_write(HMC6352_I2C_WRITE_ADDRESS) == 0 );
	_ASSERT( i2c_write(HMC6352_EXIT_CALIBRATION_COMMAND) == 0 );
	i2c_stop(); 

	DelayMs(15); // 14ms
}

BOOL HMCGet(int *nAngle)
{
	i2c_start();
	_ASSERT( i2c_write(HMC6352_I2C_WRITE_ADDRESS) == 0 );
	_ASSERT( i2c_write(HMC6352_GET_DATA_COMMAND) == 0 );
	i2c_stop(); 

	DelayMs(7); // 6ms

	i2c_start();
	_ASSERT( i2c_write(HMC6352_I2C_READ_ADDRESS) == 0 );
	*nAngle = i2c_read(1);
	*nAngle <<= 8;
	*nAngle |= i2c_read(0);
	i2c_stop(); 

	return TRUE;
}
