#include "compiler.h"
#include "Delay.h"
#include "Usart.h"
#include "hmc6352.h"

//__CONFIG(LVPDIS & BORDIS & UNPROTECT & MCLRDIS & PWRTEN & WDTDIS & XT);
__CONFIG(LVPDIS & BORDIS & UNPROTECT & MCLRDIS & PWRTEN & WDTDIS & INTIO);
unsigned int timer = 0; // [4ms step]

void interrupt isr(void)
{
	if ((TMR2IE)&&(TMR2IF)) {
		// RTC
		if (timer > 0)
			timer--;
		TMR2IF=0;
	}

}

void Init(void)
{
	// timer 2 (RTC)
	PIE1	= 0b00000011;	// TMR2 to PR2 match
	INTCON	= 0b01000000;	// peripheral int.
	T2CON	= 0b00000110;	// post =1, enable, pre = 16
	PR2	= 0b11111001;  		// 249+1

	ei();	// Global interrupts enabled
}

#define flagMAGX 1
#define flagMAGY 2
#define flagRAWX 4
#define flagRAWY 8
#define flagHEAD 16

void main()
{
	unsigned char ch, i, run=0, runmode=flagHEAD;
	DelayMs(250);     
	DelayMs(250);    
	Init();
	TRISD = 0;
	USARTInit();
	_ASSERT( HMCInit() );

	while (1)
	{
		if (!run && (timer==0))
		{
			timer = 1000;
			USARTPutROMString("Ready\r\n");
		}
		
		if (run && (timer==0))
		{
			int nAlpha, nBeta;
			timer = 25;

			if (runmode & flagMAGX )
			{
				HMCSetMode( HMC6352_MODE_MAGNETOMETER_X );
				USARTPutROMString("x=");
				_ASSERT( HMCGet(&nAlpha) );
				USARTPutW(nAlpha);
				USARTPutROMString(" ");
			}

			if (runmode & flagMAGY )
			{
				HMCSetMode( HMC6352_MODE_MAGNETOMETER_Y );
				USARTPutROMString("y=");
				_ASSERT( HMCGet(&nAlpha) );
				USARTPutW(nAlpha);
				USARTPutROMString(" ");
			}

			if (runmode & flagRAWX )
			{
				HMCSetMode( HMC6352_MODE_RAW_X );
				USARTPutROMString("X=");
				_ASSERT( HMCGet(&nAlpha) );
				USARTPutW(nAlpha);
				USARTPutROMString(" ");
			}

			if (runmode & flagRAWY )
			{
				HMCSetMode( HMC6352_MODE_RAW_Y );
				USARTPutROMString("Y=");
				_ASSERT( HMCGet(&nAlpha) );
				USARTPutW(nAlpha);
				USARTPutROMString(" ");
			}

			if (runmode & flagHEAD )
			{
				HMCSetMode( HMC6352_MODE_HEADING );
				USARTPutROMString("h=");
				_ASSERT( HMCGet(&nAlpha) );
				USARTPutInt(nAlpha);
			}
			USARTPutROMString("\r\n");

			PORTD = i++;
		}
		if ( USARTIsGetReady() )
		{
			ch = USARTGetChar();
			USARTPutROMString("[");
			USARTPut( ch );
			USARTPutROMString("]");
			switch (ch)
			{
				case 'h': runmode ^= flagHEAD; break;
				case 'x': runmode ^= flagMAGX; break;
				case 'y': runmode ^= flagMAGY; break;
				case 'X': runmode ^= flagRAWX; break;
				case 'Y': runmode ^= flagRAWY; break;

				case 'r': run = 1-run; timer = 0; break;

				case 'c':
					USARTPutROMString("Entering calibration mode\r\n");
					HMCEnterCalibration();
					for (i=0; i<20; i++)
					{
						USARTPutROMString(".");
						DelayMs(250);
						DelayMs(250);
						DelayMs(250);
						DelayMs(250);
					}
					HMCExitCalibration();
					USARTPutROMString("Calibration finished\r\n");
					break;
					
			}
		}
	}
}

