#ifndef USART_H
#define USART_H

#define CLK 4000000
#define BAUD_RATE 9600
#define CLOCK_FREQ 4000000

#define ROM const
#define BYTE unsigned char 

#if defined(USART_USE_BRGH_LOW)
    #define SPBRG_VAL(CLK)   ( (((CLK)/BAUD_RATE)/64) - 1)
#else
    #define SPBRG_VAL(CLK)   ( (((CLK)/BAUD_RATE)/16) - 1)
#endif


#if SPBRG_VAL(CLOCK_FREQ) > 255
    #error "Calculated SPBRG value is out of range for currnet CLOCK_FREQ."
#endif

#if SPBRG_VAL(CLOCK_FREQ) < 1
    #error "Calculated SPBRG value is out of range for currnet CLOCK_FREQ."
#endif

#define RX_PIN TRISC6
#define TX_PIN TRISC7

#ifdef USART_USE_BRGH_LOW
// Low BRG speed
#define USARTInit(CLK)			\
	RX_PIN = 1; TX_PIN = 1;		\
	TXSTA = 0b00100000;		\
	RCSTA = 0b10010000;		\
	SPBRG = SPBRG_VAL(CLK);
#else
// High BRG speed
/*                 hhh
#define USARTInit(CLK)			\
	TXSTA = 0b00100100;		\
    RCSTA = 0b10010000;		\
    SPBRG = SPBRG_VAL(CLK);
*/
#define USARTInit()			\
	RX_PIN = 1; TX_PIN = 1;		\
	RCSTA = 0b10010000;		\
	TXSTA = 0b00100100;		\
	SPBRG = SPBRG_VAL(CLK);		
#endif



#define USARTIsGetReady()   (RCIF)
#define USARTGET()          (RCREG)

unsigned char USARTGetChar(void);
unsigned char USARTGetByte(void);
void USARTPutInt(signed int i);
void USARTPutDW(unsigned long i);
void USARTPutW(unsigned int i);
void USARTPutROMString(char const *str);
void USARTPut(BYTE c);
BYTE USARTGet(void);
void USARTPutString(BYTE *s);
void USARTPutROMString(char const * str);
BYTE USARTGetString(char *buffer, BYTE bufferLen);
void USARTHex(BYTE c);
void USARTAssert(char const *str, int line);

#define dbg(a) USARTPutROMString(a)
#define _ASSERT(a) if (!(a)) { USARTAssert( __FILE__, __LINE__ );}
#endif